/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_OBJECTSTRUCT_H
#define DPC_IMPL_TYPE_OBJECTSTRUCT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_float.h"

namespace dpc {
struct ObjectStruct {
    ::UInt8 id{ 0U };
    ::UInt8 type{ 0U };
    ::Float relative_x{ 0.0F };
    ::Float relative_y{ 0.0F };
    ::Float relative_orientation{ 0.0F };
    ::Float absolute_velocity_x{ 0.0F };
    ::Float absolute_velocity_y{ 0.0F };
    ::Float absolute_acceleration_x{ 0.0F };
    ::Float absolute_acceleration_y{ 0.0F };
    ::Float width{ 0.0F };
    ::Float height{ 0.0F };
    ::Float length{ 0.0F };
    ::UInt8 motion_type{ 0U };
    ::UInt8 contributing_sensor{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(id);
        fun(type);
        fun(relative_x);
        fun(relative_y);
        fun(relative_orientation);
        fun(absolute_velocity_x);
        fun(absolute_velocity_y);
        fun(absolute_acceleration_x);
        fun(absolute_acceleration_y);
        fun(width);
        fun(height);
        fun(length);
        fun(motion_type);
        fun(contributing_sensor);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(id);
        fun(type);
        fun(relative_x);
        fun(relative_y);
        fun(relative_orientation);
        fun(absolute_velocity_x);
        fun(absolute_velocity_y);
        fun(absolute_acceleration_x);
        fun(absolute_acceleration_y);
        fun(width);
        fun(height);
        fun(length);
        fun(motion_type);
        fun(contributing_sensor);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("id", id);
        fun("type", type);
        fun("relative_x", relative_x);
        fun("relative_y", relative_y);
        fun("relative_orientation", relative_orientation);
        fun("absolute_velocity_x", absolute_velocity_x);
        fun("absolute_velocity_y", absolute_velocity_y);
        fun("absolute_acceleration_x", absolute_acceleration_x);
        fun("absolute_acceleration_y", absolute_acceleration_y);
        fun("width", width);
        fun("height", height);
        fun("length", length);
        fun("motion_type", motion_type);
        fun("contributing_sensor", contributing_sensor);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("id", id);
        fun("type", type);
        fun("relative_x", relative_x);
        fun("relative_y", relative_y);
        fun("relative_orientation", relative_orientation);
        fun("absolute_velocity_x", absolute_velocity_x);
        fun("absolute_velocity_y", absolute_velocity_y);
        fun("absolute_acceleration_x", absolute_acceleration_x);
        fun("absolute_acceleration_y", absolute_acceleration_y);
        fun("width", width);
        fun("height", height);
        fun("length", length);
        fun("motion_type", motion_type);
        fun("contributing_sensor", contributing_sensor);
    }

    friend bool operator==(const ::dpc::ObjectStruct& lhs, const ::dpc::ObjectStruct& rhs) noexcept
    {
        return (lhs.id == rhs.id) && (lhs.type == rhs.type) && (fabs(static_cast<double>(lhs.relative_x - rhs.relative_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.relative_y - rhs.relative_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.relative_orientation - rhs.relative_orientation)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_velocity_x - rhs.absolute_velocity_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_velocity_y - rhs.absolute_velocity_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_acceleration_x - rhs.absolute_acceleration_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.absolute_acceleration_y - rhs.absolute_acceleration_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.width - rhs.width)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.height - rhs.height)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.length - rhs.length)) < DBL_EPSILON) && (lhs.motion_type == rhs.motion_type) && (lhs.contributing_sensor == rhs.contributing_sensor);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_OBJECTSTRUCT_H
